#include "stm32f10x.h"

uint8_t waitAndRead(uint8_t withTimeout);
void init_UART();
void init_Timer();
void sendByte(uint8_t c);
volatile uint8_t timeout = 0;
void delayMs(uint8_t d);

void init_UART(){
  
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_USART1, ENABLE);
  
  GPIO_InitTypeDef GPIO_InitStructure;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
  USART_InitTypeDef USART_InitStructure;
  USART_InitStructure.USART_BaudRate = 9600;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
  USART_Init(USART1, &USART_InitStructure);
  USART_Cmd(USART1, ENABLE);

}


void init_Timer(){

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
	
	TIM_TimeBaseInitTypeDef TIM_InitStruct;
	TIM_InitStruct.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_InitStruct.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_InitStruct.TIM_Period = 9;
	TIM_InitStruct.TIM_Prescaler = 7199;
	TIM_TimeBaseInit(TIM2, &TIM_InitStruct);
	
	TIM_ClearFlag(TIM2, TIM_FLAG_Update);
	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
	TIM_Cmd(TIM2, ENABLE);
	
	NVIC_InitTypeDef NVIC_InitStruct;
	NVIC_InitStruct.NVIC_IRQChannel = TIM2_IRQn;
	NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
	NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStruct.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&NVIC_InitStruct);
}

void TIM2_IRQHandler() {
	if (TIM_GetITStatus(TIM2, TIM_IT_Update)!=RESET) {
	  	TIM_ClearFlag(TIM2, TIM_FLAG_Update);
		timeout = 1;
	}
}

int main() {
	init_UART();
	init_Timer();
	uint8_t oldControl = 0;
	while(1) {
		uint8_t ret = 0;
		uint16_t val = 0;
		for (uint8_t i=0; i<14; i++) {
			val <<= 1;
			ret = waitAndRead(i != 0);
			if (ret == 2) break;
			val |= ret;
		}
		if (ret == 2) continue;
		uint8_t command = val & 0x3f;
		uint8_t address = (val >> 6) & 0x1f;
		uint8_t control = (val >> 11) & 1;
		sendByte(command);
		sendByte(address);
		sendByte(control != oldControl);
		oldControl = control;
	}

}

uint8_t waitAndRead(uint8_t withTimeout) {
	delayMs(1);
	uint8_t state = GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0);
	uint8_t oldState = state;
	if (withTimeout) {
		TIM_SetCounter(TIM2, 0);
		timeout = 0;
		while ((state == oldState) && (timeout == 0)) state = GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0);		
		if (timeout == 1) return 2;
	} else {
		while (state == oldState) state = GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0);
	}
	if (state) return 0; else return 1;
}

void sendByte(uint8_t c) {
	while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET);
	USART_SendData(USART1, c);
}

void delayMs(uint8_t d) {
	TIM_SetCounter(TIM2, 0);
	timeout = 0;
	while (timeout == 0);
}
