#include "stm32f10x.h"

enum SoftwareUartStateRx {IDLE_RX, RECEIVE, RECEIVE_STOP};
enum SoftwareUartStateTx {IDLE_TX, TRANSMIT, TRANSMIT_STOP};
volatile enum SoftwareUartStateRx softwareUartStateRx = IDLE_RX;
volatile enum SoftwareUartStateTx softwareUartStateTx = IDLE_TX;
volatile uint8_t cyclesCounterRx, cyclesCounterTx;
volatile uint8_t softwareUartBitsToSend;
volatile uint16_t softwareUartOutputData;
volatile uint8_t softwareUartInputData;
volatile uint8_t softwareUartNewData = 0;

#define PORT_RX GPIOA //RX port
#define PIN_RX GPIO_Pin_0 //pin of RX port
#define PORT_TX GPIOA //TX port
#define PIN_TX GPIO_Pin_1 //pin of TX port
#define RCC_TX RCC_APB2Periph_GPIOA
#define RCC_RX RCC_APB2Periph_GPIOA

#define F_CPU 72000000UL
#define BAUDRATE 9600UL

void init_GPIO();
void init_Timer();

void init_Timer() {
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
	
	TIM_TimeBaseInitTypeDef TIM_InitStruct;
	TIM_InitStruct.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_InitStruct.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_InitStruct.TIM_Period = F_CPU / BAUDRATE / 3 - 1;
	TIM_InitStruct.TIM_Prescaler = 0;
	TIM_TimeBaseInit(TIM2, &TIM_InitStruct);
	
	TIM_ClearFlag(TIM2, TIM_FLAG_Update);
	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
	TIM_Cmd(TIM2, ENABLE);
	
	NVIC_InitTypeDef NVIC_InitStruct;
	NVIC_InitStruct.NVIC_IRQChannel = TIM2_IRQn;
	NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
	NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStruct.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&NVIC_InitStruct);

}

void init_GPIO() {
	RCC_APB2PeriphClockCmd(RCC_RX | RCC_TX, ENABLE);
	
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin = PIN_RX;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(PORT_RX, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin = PIN_TX;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(PORT_TX, &GPIO_InitStructure);

	GPIO_SetBits(PORT_TX, PIN_TX);
}

void TIM2_IRQHandler() {
	if (TIM_GetITStatus(TIM2, TIM_IT_Update)!=RESET) {
	  	TIM_ClearFlag(TIM2, TIM_FLAG_Update);

		static uint8_t bitsToReceive;
		static uint8_t inData;
		
		if (softwareUartStateTx == TRANSMIT) {
			if (--cyclesCounterTx == 0) {
				if(softwareUartOutputData & 1) GPIO_SetBits(PORT_TX, PIN_TX); else GPIO_ResetBits(PORT_TX, PIN_TX); //data bit
				softwareUartOutputData >>= 1;
				cyclesCounterTx = 3;
				if (--softwareUartBitsToSend == 0) {
					softwareUartStateTx = TRANSMIT_STOP;
				}
			}
		}
		
		if (softwareUartStateTx == TRANSMIT_STOP) {
			if (--cyclesCounterTx == 0) {
				softwareUartStateTx = IDLE_TX;
			}
		}				
		
		if ((softwareUartStateRx == IDLE_RX) && !(GPIO_ReadInputDataBit(PORT_RX, PIN_RX))) {
			cyclesCounterRx = 4;
			softwareUartStateRx = RECEIVE;
			bitsToReceive = 8;
			inData = 0;
		}
		
		if (softwareUartStateRx == RECEIVE) {
			if (--cyclesCounterRx == 0) {
				inData >>= 1;
				if (GPIO_ReadInputDataBit(PORT_RX, PIN_RX)) inData |= 0x80;
				cyclesCounterRx = 3;
				if (--bitsToReceive == 0) {
					softwareUartStateRx = RECEIVE_STOP;
					softwareUartInputData = inData;
					softwareUartNewData = 1;
					//processIncomingData();
				}
			}			
		}
		
		if (softwareUartStateRx == RECEIVE_STOP) {
			if (--cyclesCounterRx == 0) {
				softwareUartStateRx = IDLE_RX;
			}
		}
		
	}
}

void softwareUartSend(uint16_t data) {
	while(softwareUartStateTx != IDLE_TX);
	cyclesCounterTx = 1;
	softwareUartBitsToSend = 10;
	softwareUartOutputData = (data << 1) | 0x200;
	softwareUartStateTx = TRANSMIT;
}

int main(void)
{
	init_Timer();
	init_GPIO();

	while(1)
	{
		if(softwareUartNewData) {
			softwareUartNewData = 0;
			softwareUartSend(softwareUartInputData + 1);
		}
    }
}
