#include "lcd.h"

uint8_t switchedTo4BitMode = 0;
uint8_t checkBusyFlag = 0;

void lcdWriteCommand(uint8_t command) {
	if (switchedTo4BitMode){
  		checkBusyFlag = 1;
		lcdWrite(command, 0);
		checkBusyFlag = 0;
		lcdWrite(command << 4, 0);
	}else {
		lcdWrite(command, 0);
	}
}

void lcdWriteData(uint8_t data) {
	if (switchedTo4BitMode){
		checkBusyFlag = 1;
		lcdWrite(data, 1);
		checkBusyFlag = 0;
		lcdWrite(data << 4, 1);
	}else {
		lcdWrite(data, 1);
	}
}

uint8_t lcdReadAddress() {
	if (switchedTo4BitMode){
	  return ((lcdRead(0) & 0xf0) | (lcdRead(0) >> 4));
	}else{
	  return lcdRead(0);
	}
}

uint8_t lcdReadData(){
	if (switchedTo4BitMode){
	  return ((lcdRead(1) & 0xf0) | (lcdRead(1) >> 4));
	}else{
	  return lcdRead(1);
	}
}

void lcdWrite(uint8_t command, uint8_t rs) {
	if (checkBusyFlag) while (lcdReadAddress() & 0x80) Delay(10);
	
	//R/W low, write mode
	GPIO_ResetBits(LCD_CONTROL_PORT, LCD_RW);
	Delay(10);
	
	//configure port for write
	GPIO_InitTypeDef GPIO_InitStructure;
#ifdef BITS4
	GPIO_InitStructure.GPIO_Pin = 0xf0;
#else
	GPIO_InitStructure.GPIO_Pin = 0xff;
#endif
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);
	
	//R/S
	if (rs) GPIO_SetBits(LCD_CONTROL_PORT, LCD_RS); else GPIO_ResetBits(LCD_CONTROL_PORT, LCD_RS);
	
	//data/command
#ifdef BITS4
	GPIO_ResetBits(LCD_DATA_PORT, ~command & 0xf0);
	GPIO_SetBits(LCD_DATA_PORT, command & 0xf0);
#else
	GPIO_ResetBits(LCD_DATA_PORT, ~command);
	GPIO_SetBits(LCD_DATA_PORT, command);
#endif
	//EN high
	GPIO_SetBits(LCD_CONTROL_PORT, LCD_EN);
	//wait 1 us
	Delay(2);
	//EN low
	GPIO_ResetBits(LCD_CONTROL_PORT, LCD_EN);
	Delay(2);
}

uint8_t lcdRead(uint8_t rs) {
	//configure port for read
	GPIO_InitTypeDef GPIO_InitStructure;
#ifdef BITS4
	GPIO_InitStructure.GPIO_Pin = 0xf0;
#else
	GPIO_InitStructure.GPIO_Pin = 0xff;
#endif
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);
	
	//R/S
	if (rs) GPIO_SetBits(LCD_CONTROL_PORT, LCD_RS); else GPIO_ResetBits(LCD_CONTROL_PORT, LCD_RS);
	//R/W high
	GPIO_SetBits(LCD_CONTROL_PORT, LCD_RW);
	//EN high
	GPIO_SetBits(LCD_CONTROL_PORT, LCD_EN);
	//wait 1 us
	Delay(2);
	//command
	uint8_t data = GPIO_ReadInputData(LCD_DATA_PORT);
	//EN low
	GPIO_ResetBits(LCD_CONTROL_PORT, LCD_EN);
	Delay(2);
	return data;
}

void lcdInit() {
	init_GPIO();
	delayMs(15);
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_8BITS); // set interface - 8-bit
	delayMs(4);
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_8BITS); // set interface - 8-bit
	delayMs(1);
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_8BITS); // set interface - 8-bit
#ifdef BITS4
	delayMs(1);
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET); // set interface - 4-bit
	delayMs(1);
	switchedTo4BitMode = 1;
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_2LINES); // set interface - 4-bit, 2 lines, 5x8 font
#else
	checkBusyFlag = 1;
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_8BITS | LCD_OPTION_2LINES); // set interface - 8-bit, 2 lines, 5x8 font
#endif
	lcdWriteCommand(LCD_COMMAND_ON); // off - nothing to be on
	lcdWriteCommand(LCD_COMMAND_ON | LCD_OPTION_DISPLAY); // display on
	lcdWriteCommand(LCD_COMMAND_ENTRY_MODE | LCD_OPTION_INCREMENT); // scroll
	lcdWriteCommand(LCD_COMMAND_CLEAR); // clear
}

void lcdString(char * buf) {
	while (*buf != 0) {
		lcdWriteData(*buf);
		buf++;
	}
}

void init_GPIO() {
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB, ENABLE);
  
  GPIO_InitTypeDef GPIO_InitStructure;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = LCD_EN | LCD_RW | LCD_RS;
  GPIO_Init(LCD_CONTROL_PORT, &GPIO_InitStructure);
}

void Delay(volatile unsigned int delay)
{
  for(;delay>0;delay--);
}

void delayMs(uint8_t d) {
  for (uint8_t i = 0; i < d; i++) Delay(12000);
}
